package algorithms;

import java.util.ArrayList;
import java.util.List;
import objects.ClassifiableList;
import objects.Template;

public class ClassifierLinear extends Classifier {

    private List<Template> centroids;

    @Override
    public void importTemplatesGallery(List<Template> templatesGallery) {
        centroids = new ArrayList();
        List<List<Template>> templatesByClass = new ClassifiableList(templatesGallery).splitBySubject();
        for (List<Template> templatesOfClass : templatesByClass) {
            if (templatesOfClass.isEmpty()) {
                System.out.println("empty class");
            } else {
                Template centroid = templatesOfClass.get(0);
                double minSum = Double.MAX_VALUE;
                for (Template centroidCandidate : templatesOfClass) {
                    double sum = 0;
                    for (Template template : templatesOfClass) {
                        sum += getDistance(template, centroidCandidate);
                    }
                    if (minSum > sum) {
                        minSum = sum;
                        centroid = centroidCandidate;
                    }
                }
                centroids.add(centroid);
            }
        }
    }

    @Override
    public String classify(Template templateProbe) {
        Template closestCentroid = centroids.get(0);
        double minDistance = Double.MAX_VALUE;
        for (Template centroid : centroids) {
            double distance = getDistance(centroid, templateProbe);
            if (minDistance > distance) {
                minDistance = distance;
                closestCentroid = centroid;
            }
        }
        return closestCentroid.getSubject();
    }
}
