package methods;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;
import algorithms.Classifier;
import algorithms.Classifier1NN;
import algorithms.DistancePosesL1;
import algorithms.DistanceTemplatesL1;
import algorithms.DistanceTemplatesMahalanobis;
import algorithms.MotionLoader;
import com.mkobos.pca_transform.PCA;
import java.util.ArrayList;
import java.util.List;
import objects.ClassifiableList;
import objects.Feature;
import objects.Motion;
import objects.Parameter;
import objects.Sample;

public class Method_PCALDA extends Method {

    private final Classifier classifier = new Classifier1NN();
    private final int length = 150;

    public Method_PCALDA(MotionLoader motionLoader) {
        super(motionLoader);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {

        // TRANSFORMATION MATRIX
        List<Sample> samples = new ArrayList();
        for (Motion motion : motionsLearning) {
            motion.adjust(length);
            samples.add(new Sample(motion.getIdentifier(), motion.getVector()));
        }
        Matrix meanSampleMatrix = getMeanMatrix(samples);
        int dimension = samples.get(0).getDimension();
        Matrix samplesMatrix = new Matrix(samples.size(), dimension);
        for (int d = 0; d < dimension; d++) {
            for (int s = 0; s < samples.size(); s++) {
                samplesMatrix.set(s, d, samples.get(s).getParameterValue(d, 0) - meanSampleMatrix.get(d, 0));
            }
        }
        Matrix SigmaB = getBetweenClassScatterMatrix(samples);
        Matrix SigmaW = getWithinClassScatterMatrix(samples);
        Matrix PhiPCA = new PCA(samplesMatrix).getEigenvectorsMatrix().getMatrix(0, dimension - 1, 0, new ClassifiableList(samples).splitBySubject().size() - 1);
        Matrix PhiLDA = new EigenvalueDecomposition((PhiPCA.transpose().times(SigmaW).times(PhiPCA)).inverse().times(PhiPCA.transpose().times(SigmaB).times(PhiPCA))).getV();
        classifier.setTransformationMatrix(PhiPCA.times(PhiLDA));
        setClassifier(classifier);

        // DISTANCE FUNCTION
//        DistanceTemplatesL1 distanceTemplates = new DistanceTemplatesL1();//
//        distanceTemplates.setDistancePoses(new DistancePosesL1());//
        DistanceTemplatesMahalanobis distanceTemplates = new DistanceTemplatesMahalanobis();//
        distanceTemplates.setMatrix(getTotalScatterMatrix(extractTemplates(motionsLearning)).inverse());//
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) {
        motion.adjust(length);
        double[][] data = getClassifier().getTransformationMatrix().transpose().times(motion.getVector()).getArray();
        List<Feature> features = new ArrayList();
        for (int d = 0; d < data.length; d++) {
            features.add(new Feature(new Parameter[]{new Parameter(String.valueOf(d), data[d][0])}));
        }
        return features;
    }
}
